# installation used by quickinstaller

from Products.ExternalMethod.ExternalMethod import ExternalMethod
from Products.CMFCore.utils import getToolByName
from Products.Archetypes import listTypes
from Products.Archetypes.Extensions.utils import installTypes, install_subskin
from Products.PloneFormMailer import PROJECTNAME, GLOBALS
from Products.PloneFormMailer.config import FORM_ENCTYPE
from StringIO import StringIO
from Products.CMFDynamicViewFTI.migrate import migrateFTIs

out = StringIO()

def install(self):
    classes=listTypes(PROJECTNAME)
    installTypes(self, out,
                 classes,
                 PROJECTNAME)
    install_subskin(self, out, GLOBALS)

    _setWorkflow(self)
    _migrateSchema(self)
    _fixProperties(self)
    _settableAsIndex(self)
    _migrateFTI(self)
    return out.getvalue()


def _setWorkflow(self):
    wft = getToolByName(self,'portal_workflow')
    iwf = ExternalMethod('temp',
                       'temp',
                       PROJECTNAME+'.'+'PloneFormMailer_workflow',
                       'createPloneFormMailer_workflow') 
    wf = iwf(self,'PloneFormMailer_workflow')
    wft._setObject('PloneFormMailer_workflow', wf)
    wft.setChainForPortalTypes( ['PloneFormMailer'], wf.getId())
    print >> out, "Ploneformmailer workflow set."

def _migrateSchema(self):
    at = getToolByName(self, 'archetype_tool')
    class dummy:
        form = {}
    dummyRequest = dummy()
    dummy.form['PloneFormMailer.PloneFormMailer'] = 1
    at.manage_updateSchema(update_all=1,
                           REQUEST=dummyRequest)
    print >> out, "Migrated all PFM schemas."

def _fixProperties(self):
    catalog = self.portal_catalog
    pfm_brains = catalog(Type='Form Mailer')
    print >> out, "Found %s PFM forms." % len(pfm_brains)
    for pfm_brain in pfm_brains:
        pfm = pfm_brain.getObject()
        if pfm is None:
            # ignore stale catalog
            continue
        form = pfm.form
        form.enctype = FORM_ENCTYPE
        form.unicode_mode = True
    print >> out, "Reset enctype and unicode_mode in those forms."

def _settableAsIndex(self):
    props_tool = getToolByName(self, 'portal_properties')
    siteProperties = props_tool.site_properties
    defaultPageTypesProp = siteProperties.getProperty(
        'default_page_types', None)
    defaultPageTypes = list(defaultPageTypesProp)
    if defaultPageTypesProp == None:
        out.write("No default_page_types property found, so "
                  "not changing. Probably plone<2.1.\n")
        return
    changed = False
    type_ = 'PloneFormMailer'
    if type_ not in defaultPageTypes: 
        defaultPageTypes.append(type_)
        print >> out, ("Added %s to default page " % type_ +
                       "types (=addable as index pages)")
        changed = True
    if changed:
        siteProperties.manage_changeProperties(default_page_types 
                                               = defaultPageTypes) 

def _migrateFTI(self):
    migrated = migrateFTIs(self, product=PROJECTNAME, fti_meta_type='PloneFormMailer')
    print >> out, "Switched to DynamicViewFTI: %s" % ', '.join(migrated)    
 