#------------------------------------------------------------------------------
# Name:         schema.py
# Purpose:      The Archetypes Schema description for PloneFormMailer
#
# Author:       Jens Klein <jens@bluedynamics.com>
#
# RCS-ID:       $Id: schema.py 24554 2006-06-07 17:31:45Z limi $
# Copyright:    (c) 2004-2006 by BlueDynamics, Klein & Partner KEG, Austria
# Licence:      GNU General Public Licence (GPL) Version 2 or later
#------------------------------------------------------------------------------

from config import *
if HAS_LINGUA_PLONE:
    from Products.LinguaPlone.public import *
else:
    from Products.Archetypes.atapi import *
from Products.TALESField import TALESString, TALESLines
from Products.TemplateFields import ZPTField

# for PloneFormMailer we want to have the description in the edit view
# just like CMF
PFMschema = BaseSchema.copy()
PFMschema['description'].isMetadata = False
PFMschema['description'].schemata = 'default'

PFMschema += Schema((
    # before script is called before page template is rendered
    # reccomended in before script is check form.submitted
    # variable and do nothing when is set to 1
    StringField('before_script',
                required=0,
                default='',
                vocabulary = '_get_scripts_list',
                widget=SelectionWidget(label='Before script',
                                       description='This script will be called before template is rendered.',
                                       label_msgid='label_before_script',
                                       description_msgid='help_before_script',
                                       format='select',
                                       i18n_domain='pfm'),
                schemata='Web Form',
    ),

    #text prepended to list of fields
    TextField('form_pre',
              accessor='getFormPre',
              mutator='setFormPre',
        widget=RichWidget(description = "Text prepended to the web-form.",
            description_msgid = "help_formmailer_form_pre",
            label = "Text (prepended)",
            label_msgid = "label_formmailer_form_pre",
            i18n_domain='pfm',
            rows = 12
        ),
        default_content_type='text/structured',
        allowable_content_types=('text/structured',
                                'text/restructured',
                                'text/html',
                                'text/plain'),
        schemata='Web Form',
        default_output_type='text/html'
    ),
    #text appended to list of fields
    TextField('form_post',
              accessor='getFormPost',
              mutator='setFormPost',
        widget=RichWidget(description = "Text appended to the web-form.",
            description_msgid = "help_formmailer_form_post",
            label = "Text (appended)",
            label_msgid = "label_formmailer_form_post",
            i18n_domain='pfm',
            rows = 12),
        default_content_type='text/structured',
        allowable_content_types=('text/structured',
                                'text/restructured',
                                'text/html',
                                'text/plain'),
        schemata='Web Form',
        default_output_type='text/html'
    ),


    #label for send button
    LinesField('form_buttons',
               accessor='getFormButtons',
               mutator='setFormButtons',
        widget=LinesWidget(description = 'Definition of the buttons. '
            'format: value|name|type|css-class.',
            description_msgid = "help_formmailer_form_buttons",
            label = 'Buttons',
            label_msgid = "label_formmailer_form_button_submit",
            i18n_domain='pfm',
        ),
        default=("""Submit|submit|submit|context"""),
        schemata='Web Form',
        required=1,

    ),
    # name of the recipient
    TALESString('recipient_name',
                accessor='getRecipientName',
                mutator='setRecipientName',
        widget=StringWidget(description = 'The full name of the recipient '
            'of the mailed form. TALES Expression. If you don\'t know '
            'TALES expressions just enter string:YOURTEXT.',
            description_msgid = "help_formmailer_recipient_fullname",
            label = 'Recipient\'s full name',
            label_msgid = "label_formmailer_recipient_fullname",
            i18n_domain='pfm',
        ),
        languageIndependent=1,
        validators=('talesvalidator',),
        schemata='Mail Headers',
        default='',
    ),

    #e-mail adress of the recipient of the form-data
    TALESString('recipient_email',
                accessor='getRecipientEmail',
                mutator='setRecipientEmail',
        widget=StringWidget(description = 'The recipients e-mail address. '
            'TALES Expression. If you don\'t know TALES expressions just '
            'enter string:YOURTEXT.',
            description_msgid = "help_formmailer_recipient_email",
            label = 'Recipient\'s e-mail address',
            label_msgid = "label_formmailer_recipient_email",
            i18n_domain='pfm',
        ),
        languageIndependent=1,
        validators=('talesvalidator',),
        schemata='Mail Headers',
        default='',
    ),

    #additional recipients email-adresses by CC
    TALESLines('cc_recipients',
               accessor='getCCRecipients',
               mutator='setCCRecipients',
        widget=LinesWidget(
            description = 'E-mail addresses which receive a carbon copy. '
            'TALES Expression. If you don\'t know TALES expressions just '
            'enter string:YOURTEXT.',
            description_msgid = "help_formmailer_cc_recipients",
            label = 'Cc recipients',
            label_msgid = "label_formmailer_cc_recipients",
            i18n_domain='pfm',
        ),
        languageIndependent=1,
        validators=('talesvalidator',),
        schemata='Mail Headers',
        required=0,
        default=[],
    ),


    #additional recipients email-adresses by BCC
    TALESLines('bcc_recipients',
               accessor='getBCCRecipients',
               mutator='setBCCRecipients',
        widget=LinesWidget(
            description = 'E-mail addresses which receive a blind carbon copy. '
            'TALES Expression. If you don\'t know TALES expressions just '
            'enter string:YOURTEXT.',
            description_msgid = "help_formmailer_bcc_recipients",
            label = 'Bcc recipients',
            label_msgid = "label_formmailer_bcc_recipients",
            i18n_domain='pfm',
        ),
        languageIndependent=1,
        validators=('talesvalidator',),
        schemata='Mail Headers',
        required=0,
        default=[],
    ),

    #header lines in formatting KEY: VALUE
    LinesField('additional_headers',
               accessor='getAdditionalHeaders',
               mutator='setAdditionalHeaders',
        widget=LinesWidget(description = 'Additional e-mail-header lines. '
            'Only fill in if you aware of RFC822.',
            description_msgid = "help_formmailer_additional_headers",
            label = 'Additional headers',
            label_msgid = "label_formmailer_additional_headers",
            i18n_domain='pfm',
        ),
        languageIndependent=1,
        schemata='Mail Headers'
    ),


    #subject of the message
    TALESString('mailSubject',
                accessor='getMailSubject',
                mutator='setMailSubject',
        default = DEFAULT_MAILTEMPLATE_SUBJECT,
        widget=StringWidget(description = 'Subject line of message. '
            'TALES Expression. If you don\'t know TALES expressions just '
            'enter string:YOURTEXT.',
            description_msgid = "help_formmailer_subject",
            label = 'Subject',
            label_msgid = "label_formmailer_subject",
            i18n_domain='pfm',
        ),
        languageIndependent=1,
        validators=('talesvalidator',),
        schemata='Mail Message',
        required=0,
    ),

    #text introduces the body before field-listing starts
    TextField('body_pre',
              accessor='getBodyPre',
              mutator='setBodyPre',
        widget=TextAreaWidget(description = 'Text prepended to fields listed '
            'in mail-body',
            description_msgid = "help_formmailer_body_pre",
            label = 'Body (prepended)',
            label_msgid = "label_formmailer_body_pre",
            i18n_domain='pfm',
        ),
        languageIndependent=1,
        schemata='Mail Message'
    ),

    #text in e-mail after field list
    TextField('body_post',
        accessor='getBodyPost',
        mutator='setBodyPost',
        widget=TextAreaWidget(description = 'Text appended to fields listed '
            'in mail-body',
            description_msgid = "help_formmailer_body_post",
            label = 'Body (appended)',
            label_msgid = "label_formmailer_body_post",
            i18n_domain='pfm',
        ),
        languageIndependent=1,
        schemata='Mail Message'
    ),

    #text used as the footer of e-mail (do not include delimiter)
    TextField('body_footer',
        accessor='getBodyFooter',
        mutator='setBodyFooter',
        widget=TextAreaWidget(description = 'Text used as the footer at '
            'bottom, delimited from the body by a dashed line.',
            description_msgid = "help_formmailer_body_footer",
            label = 'Body (signature)',
            label_msgid = "label_formmailer_body_footer",
            i18n_domain='pfm',
        ),
        languageIndependent=1,
        schemata='Mail Message'
    ),

    #title of 'Sent Response'
    StringField('sent_title',
                accessor='getSentTitle',
                mutator='setSentTitle',
        widget=StringWidget(description = 'The title of the page displayed '
            'after successful sending. If left blank the primary title '
            'field is used.',
            description_msgid = "help_formmailer_sent_title",
            label = 'Title of the thank-you page',
            label_msgid = "label_formmailer_sent_title",
            i18n_domain='pfm',
        ),
        schemata='Thank-You Page'
    ),

    #message for 'Sent Response'
    TextField('sent_message',
              accessor='getSentMessage',
              mutator='setSentMessage',
        widget=RichWidget(description = 'The message of the page displayed '
            'after successful sending.',
            description_msgid = "help_formmailer_sent_msg",
            label = 'Body text for the thank-you page',
            label_msgid = "label_formmailer_sent_msg",
            i18n_domain='pfm',
        ),
        default_content_type='text/structured',
        allowable_content_types=('text/structured',
                                'text/restructured',
                                'text/html',
                                'text/plain'),
        schemata='Thank-You Page',
        default_output_type='text/html'
    ),

    #ignore sent_title, sent_message and redirect to another page
    TALESString('sent_redirect',
                accessor='getSentRedirect',
                mutator='setSentRedirect',
        widget=StringWidget(description = 'Redirect to a custom page '
            'is possible too (instead of standard thank-you page).'
            'TALES Expression. If you don\'t know TALES expressions just '
            'enter string:YOURTEXT.',
            description_msgid = "help_formmailer_sent_redirect",
            label = 'Redirect to (sent)',
            label_msgid = "label_formmailer_sent_redirect",
            i18n_domain='pfm',
        ),
        schemata='Thank-You Page',
        validators=('talesvalidator',),
        required=0,
        default='',
    ),

    #text introduces the body before field-listing starts
    ZPTField('body_pt',
        accessor='getBodyPT',
        mutator='setBodyPT',
        default = DEFAULT_MAILTEMPLATE_BODY,
        widget=TextAreaWidget(description = 'This is a Zope Page Template '
            'used for rendering of the mail-body. You don\'t need to modify '
            'it, but if you know TAL (Zope\'s Template Attribute Language) '
            'you have the full power to customize your outgoing mails.',
            description_msgid = "help_formmailer_body_pt",
            label = 'Mail Body Template',
            label_msgid = "label_formmailer_body_pt",
            i18n_domain='pfm',
            rows = 30,
        ) ,
        languageIndependent=1,
        validators=('zptvalidator',),
        schemata='Advanced'
    ),
              
    StringField('body_type',
                accessor='getBodyType',
                mutator='setBodyType',
                default='html',
                vocabulary = MIME_LIST,
                widget=SelectionWidget(description = 'Set the mime-type of the mail-body. '
                                       'Change this setting only if you know exactly what you are doing. '
                                       'Leave it blank for default behaviour.',
                                       description_msgid = "help_formmailer_body_type",
                                       label = 'Mail-Body Type',
                                       label_msgid = "label_formmailer_body_type",
                                       i18n_domain='pfm',
                                       ),
                languageIndependent=1,
                schemata='Advanced'
    ),

    #gpg key-id
    StringField('gpg_keyid',
                accessor='getGPGKeyId',
                mutator='setGPGKeyId',
        widget=StringWidget(description = 'Give your key-id, e-mail or '
            'whatever works to match a public key from current keyring. '
            'It will be used to encrypt the '
            'message body. You need to have a working "GNU Privacy Guard" binary '
            'installed on the machine running zope! Test this feature before going public!',
            description_msgid = "help_formmailer_key_id",
            label = 'GPG Key ID',
            label_msgid = "label_formmailer_key_id",
            i18n_domain='pfm',
        ),
        languageIndependent=1,
        schemata='Advanced'
    ),

##        #gpg keyserver
##        StringField('gpg_keyserver',
##            widget=StringWidget(description = 'Name a Keyserver, where the '
##                'above key can be found. Not needed if it is already in current'
##                'keyring or a public key is given',
##                description_msgid = "help_formmailer_gpg_keyserver",
##                label = 'Keyserver',
##                label_msgid = "label_formmailer_gpg_keyserver",
##            ),
##            schemata='Encryption'
##        ),
##
##
##        #gpg public key
##        TextField('gpg_public',
##            widget=TextAreaWidget(description = 'Paste your GPG/PGP public key in '
##                'here.',
##                description_msgid = "help_formmailer_gpg_public",
##                label = "Public Key",
##                label_msgid = "label_formmailer_gpg_public",
##                rows = 12),
##            default_content_type='text/plain',
##            allowable_content_types=('text/plain'),
##            schemata='Encryption',
##            default_output_type='text/html'
##        ),

    #action to be executed after successful validation
    StringField('cpyaction',
                accessor='getCPYAction',
                mutator='setCPYAction',
        widget=StringWidget(description = 'Set an alternate controller '
            'python script (cpy). Change this setting only if you know '
            'exactly what you are doing. Leave it blank for default behaviour. ' 
            'TALES Expression. If you don\'t know TALES expressions just enter ' 
            'string:YOURSCRIPTNAME',
            description_msgid = "help_formmailer_cpyaction",
            label = 'Controller Script',
            label_msgid = "label_formmailer_cpyaction",
            i18n_domain='pfm',
        ),
        languageIndependent=1,
        schemata='Advanced'
    ),




),
)
